{-|
Module      : Gargantext.Core.Worker
Description : Asynchronous worker logic
Copyright   : (c) CNRS, 2024
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

-}


module Gargantext.Core.Worker where


import Async.Worker.Broker.Redis (RedisBroker)
import Async.Worker.Broker.Types (BrokerMessage, toA, getMessage)
import Async.Worker qualified as Worker
import Async.Worker.Types qualified as Worker
import Async.Worker.Types (HasWorkerBroker)
import Data.Text qualified as T
import Gargantext.API.Admin.Auth (forgotUserPassword)
import Gargantext.API.Admin.Auth.Types (ForgotPasswordAsyncParams(..))
import Gargantext.API.Admin.Types (HasSettings)
import Gargantext.Core.Worker.Jobs
import Gargantext.Core.Worker.Jobs.Types (Job(..))
import Gargantext.Core.Worker.TOML (WorkerDefinition(..), wdToRedisConnectInfo)
import Gargantext.Database.Prelude (CmdCommon)
import Gargantext.Database.Query.Table.User (getUsersWithEmail)
import Gargantext.Prelude

  

-- | Spawn a worker with Redis broker
-- TODO:
--  - reduce size of DB pool
--  - progress report via notifications
--  - I think there is no point to save job result, as usually there is none (we have side-effects only)
--  - replace Servant.Job to use workers instead of garg API threads
withRedisWorker :: (HasWorkerBroker RedisBroker Job, HasSettings env, CmdCommon env)
                => env
                -> WorkerDefinition
                -> (Async () -> Worker.State RedisBroker Job -> IO ())
                -> IO ()
withRedisWorker env wd@(WorkerDefinition { .. }) cb = do
  case wdToRedisConnectInfo wd of
    Nothing -> panicTrace $ "worker definition: could not create redis conn info"
    Just connInfo -> do
      broker <- initializeRedisBroker connInfo
    
      let state' = Worker.State { broker
                                , queueName = _wdQueue
                                , name = T.unpack _wdName
                                , performAction = performAction env
                                , onMessageReceived = Nothing
                                , onJobFinish = Nothing
                                , onJobTimeout = Nothing
                                , onJobError = Nothing }
    
      withAsync (Worker.run state') (\a -> cb a state')


performAction :: (HasWorkerBroker b Job, HasSettings env, CmdCommon env)
              => env
              -> Worker.State b Job
              -> BrokerMessage b (Worker.Job Job)
              -> IO ()
performAction env _state bm = do
  let job' = toA $ getMessage bm
  case Worker.job job' of
    Ping -> putStrLn ("ping" :: Text)
    ForgotPasswordAsync { _fpa_args = ForgotPasswordAsyncParams { email } } -> flip runReaderT env $ do
      liftBase $ putStrLn ("forgot password: " <> email)
      us <- getUsersWithEmail (T.toLower email)
      case us of
        [u] -> forgotUserPassword u
        _ -> pure ()
    GargJob { _gj_garg_job } -> putStrLn ("Garg job: " <> show _gj_garg_job :: Text)
